; IOCOM - Public domain by Eric Auer 2004.
; WRITE and
; Receive data from serial port, give up on timeout. Forward data
; to standard output. Return useful errorlevel. Nice together
; with MODE for serial port setting and ECHO ... > COMn for ...
; Compile: http://nasm.sf.net/ "nasm -o readcom.com readcom.asm"

	org 100h	; .com file
start:	mov si,81h	; command line args
	xor ax,ax	; clear high half
cmd:	mov al,[si]	; read char from command line args
	inc si		; next char
	cmp al,9
	jz cmd		; skip tab
	cmp al,' '
	jz cmd		; skip space
	cmp al,'1'
	jb help		; all other < '1' is wrong
	cmp al,'4'
	ja help		; all other > '4' is wrong
	sub al,'1'
	mov di,ax	; port number, 0..3
; ***	jmp short getit	; ignore rest and get going

sendit:	mov al,[si]	; fetch char
	inc si		; move on to next char
	mov bx,5	; try 5 times

send2:	mov ah,1	; send char
	mov dx,di	; port, 0..3 for COM1..COM4
	push ax		; save char
	int 14h		; serial
	test ah,80h	; timeout?
	mov cx,ax	; save status
	pop ax
	jnz sendok
	dec bx		; count down attempts
	jnz send2

	mov ax,cx
	jmp err		; abort

sendok:	cmp al,13	; did we send a CR?
	jnz sendit	; else loop around
	jmp short getit
	
help:	mov ah,9	; string output
	mov dx,usage	; help text
	int 21h		; dos
	mov ax,4cffh	; exit with errorlevel 255
	int 21h		; dos

getit:	xor cx,cx	; char count
next:	mov ax,200h	; read char
	mov dx,di	; port, 0..3 for COM1..COM4
	int 14h		; serial
	test ah,80h	; timeout?
	jnz err		; exit if timeout
	inc cx
	cmp al,13	; cr received?
	jz okay		; exit then
	cmp cx,126	; close to max?
	jb nclose
	jmp okay	; *** simply exit if too many chars received
; ***	dec cx		; then limit count
nclose:	mov ah,2	; write stdout (expands tabs, checks ^c)
	cmp al,' '	; printable (not cr, lf, nul, esc, ...)?
	jae okchar	; if not, replace
	mov al,'?'	; replace by '?'
okchar:	mov dl,al	; the char
	int 21h		; dos
	jmp short next	; loop around

err:	push ax
	mov dx,errstr
	mov ah,9	; string output
	int 21h		; dos
	pop ax
	mov al,ah	; error bits (timeout will be set)
	and al,8fh	; keep timeout and low bits: 8 = frame error,
			; 4 = parity error 2 = overrun 1 = data received)
	mov ah,4ch	; exit to dos with errorlevel (128..143)
	int 21h		; dos

okay:	mov dx,crlf
	mov ah,9	; string output
	int 21h		; dos
	mov ah,4ch	; exit to dos with errorlevel (0..127)
	mov al,cl	; char count
	int 21h

errstr	db "Timeout."	; continued in next line
crlf	db 13,10,"$"	; cr, lf, end-of-dos-string
usage	db "Usage: 'IOCOM nxxxx' where n is the com port number 1..4",13,10
	db "Reads bytes from port until a timeout happens or a CR is",13,10
	db "received. Errorlevel is 128..143 if timeout. Otherwise",13,10
	db "it is the number of received bytes or at most 127.",13,10,
	db "   xxx is a string which is sent before receiving starts.",13,10
	db "   a CR will be added to that string on sending.",13,10,"$"

	; other calls: int 14.3 reads full status:
	; AH - timeout txempty txholdempty break
	;         frameerror parityerror overrunerror receiveddataready
	; AL - CD RI DSR CTS dCD dRI dDSR dCTS (d=delta, line changed)
	; (carrier detect, ring indicator, clear to send, data set ready)
